<?php

namespace FintectureWC;

use FintectureWC\Fintecture\PisClient;
use FintectureWC\Fintecture\Config\Telemetry;
use FintectureWC\Fintecture\Util\Validation;
use FintectureWC\Fintecture\Util\Crypto;
if (!\defined('ABSPATH')) {
    exit;
}
// Check that WooCommerce class exists
if (!\class_exists('WC_Payment_Gateway')) {
    return;
}
class Fintecture_Post_Payment extends \WC_Payment_Gateway
{
    /**
     * The current version of the plugin
     *
     * @since 1.0
     * @access protected
     * @var string $version The current version of the plugin.
     */
    protected $version;
    /**
     * @var bool $environment
     */
    private $environment;
    /**
     * @var string $app_id
     */
    private $app_id;
    /**
     * @var string $test_app_id
     */
    private $test_app_id;
    /**
     * @var string $app_secret
     */
    private $app_secret;
    /**
     * @var string $test_app_secret
     */
    private $test_app_secret;
    /**
     * @var string $app_private_key
     */
    private $app_private_key;
    /**
     * @var string $test_app_private_key
     */
    private $test_app_private_key;
    /**
     * @var bool $show_branding
     */
    private $show_branding;
    /**
     * @var array<PisClient> $pisClients
     */
    private $pisClients;
    /**
     * Define the core functionality of the plugin
     *
     * @since 1.0
     */
    public function __construct()
    {
        $this->version = \FINTECTURE_WC_PLUGIN_VERSION;
        $this->id = 'wc_fintecture_gateway';
        $this->icon = '';
        $this->has_fields = \true;
        // set to true to display this template : 'checkout-description.php'
        $this->method_title = \__('Fintecture Payment Gateway', 'fintecture-payment-gateway');
        $this->method_description = \__('Fintecture Payment Gateway Plug-in for WooCommerce', 'fintecture-payment-gateway');
        $this->supports = ['products'];
        // After init_settings() is called, you can get the settings and load them into variables, e.g:
        // $this->title = $this->get_option('title');
        $this->init_settings();
        // Get setting values
        $this->title = \__('Instant bank payment', 'fintecture-payment-gateway');
        $this->enabled = $this->get_option('enabled');
        $this->environment = 'yes' === $this->get_option('environment');
        $this->app_id = $this->get_option('app_id');
        $this->test_app_id = $this->get_option('test_app_id');
        $this->app_secret = $this->get_option('app_secret');
        $this->test_app_secret = $this->get_option('test_app_secret');
        $this->app_private_key = $this->get_option('app_private_key');
        $this->test_app_private_key = $this->get_option('test_app_private_key');
        $this->show_branding = 'yes' === $this->get_option('show_branding');
        $this->initPisClient();
        // This basically defines your configuration form which are then loaded with init_form_fields()
        $this->init_form_fields();
        // Webhooks for Fintecture purposes
        \add_action('woocommerce_api_fintecture_post_payment', [$this, 'post_payment']);
        \add_action('woocommerce_api_fintecture_webhook', [$this, 'webhook']);
        \add_action('woocommerce_api_fintecture_connection_test', [$this, 'connection_test']);
        \add_action('woocommerce_api_fintecture_checkout_event', [$this, 'checkout_event']);
        \add_action('woocommerce_api_fintecture_dropdown_event', [$this, 'dropdown_event']);
        if ($this->inFintectureSectionPage()) {
            \add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);
            \add_action('admin_enqueue_scripts', [$this, 'enqueue_scripts']);
            // Verify requirements
            \add_action('admin_notices', [$this, 'admin_notices_fintecture']);
        }
    }
    /**
     * Enqueues admin scripts
     *
     * @since 1.0
     */
    public function enqueue_scripts() : void
    {
        \wp_register_script('wc_gateway_fintecture_settings_js', \plugins_url('assets/js/wc-gateway-settings.js', __DIR__), ['jquery'], \false, \true);
        \wp_enqueue_script('wc_gateway_fintecture_settings_js');
        \wp_localize_script('wc_gateway_fintecture_settings_js', 'script_vars', ['connection_test_url' => \site_url('/wc-api/fintecture_connection_test'), 'success' => \__('Connection succeeded', 'fintecture-payment-gateway'), 'failed' => \__('Connection failed', 'fintecture-payment-gateway'), 'mandatory' => \__('This field is mandatory', 'fintecture-payment-gateway')]);
        \wp_enqueue_style('wc_gateway_fintecture_settings_css', \plugins_url('assets/css/wc-gateway-settings.css', __DIR__));
    }
    /**
     * Return if we are in the Fintecture configuration plugin page
     *
     * @return bool
     * @since 1.0
     * @access private
     */
    private function inFintectureSectionPage() : bool
    {
        $params = ['page' => isset($_GET['page']) ? \sanitize_text_field($_GET['page']) : '', 'tab' => isset($_GET['tab']) ? \sanitize_text_field($_GET['tab']) : '', 'section' => isset($_GET['section']) ? \sanitize_text_field($_GET['section']) : ''];
        return !empty($params['page']) && !empty($params['tab']) && !empty($params['section']) && $params['section'] === 'wc_fintecture_gateway';
    }
    /**
     * Add an error to WordPress dashboard (shoul be called in an admin notice hook)
     *
     * @param string $message Error message
     * @param string $suffix suffix to append after error message (not translated)
     * @param string $type type of notice
     * @access private
     */
    private function add_admin_error(string $message, string $suffix = '', string $type = 'error') : void
    {
        $message = 'Fintecture - ' . \__($message, 'fintecture-payment-gateway') . $suffix;
        echo '<div class="notice notice-' . $type . '"><p>' . $message . '</p></div>';
    }
    /**
     * Add an error to WooCommerce admin form
     *
     * @param string $message Error message
     * @param string $suffix suffix to append after error message (not translated)
     * @access private
     */
    private function add_settings_error(string $message, string $suffix = '') : void
    {
        $message = 'Fintecture - ' . \__($message, 'fintecture-payment-gateway') . $suffix;
        $this->add_error($message);
    }
    /**
     * Add an error in checkout page
     *
     * @param string $message Error message
     * @param string $suffix suffix to append after error message (not translated)
     * @param string $type type of notice
     * @access private
     */
    private function add_front_error(string $message, string $suffix = '', string $type = 'error') : void
    {
        $message = 'Fintecture - ' . \__($message, 'fintecture-payment-gateway') . $suffix;
        \wc_print_notice($message, $type);
    }
    /**
     * Add an error message into admin for unreadable sandbox private key
     *
     * @since 3.0
     */
    public function privateKeyErrorSandbox() : void
    {
        $this->add_admin_error('Test mode (sandbox): unable to read the Private Key. Please verify it and re-upload it.');
    }
    /**
     * Add an error message into admin for unreadable production private key
     *
     * @since 3.0
     */
    public function privateKeyErrorProduction() : void
    {
        $this->add_admin_error('Production: unable to read the Private Key. Please verify it and re-upload it.');
    }
    /**
     * Add an error message into admin for incomplete sandbox credentials
     *
     * @since 3.0
     */
    public function incompleteErrorSandbox() : void
    {
        $this->add_admin_error('Test mode (sandbox): configuration is incomplete, please verify credential fields.');
    }
    /**
     * Add an error message into admin for incomplete production credentials
     *
     * @since 3.0
     */
    public function incompleteErrorProduction() : void
    {
        $this->add_admin_error('Production: configuration is incomplete, please verify credential fields.');
    }
    /**
     * Call all checks
     */
    public function admin_notices_fintecture() : void
    {
        $this->do_curl_check();
        $this->do_openssl_check();
        $this->do_permalinks_structure_check();
        $this->do_price_decimals_check();
    }
    /**
     * Check if cURL is available on the system
     *
     * @since 1.0
     */
    public function do_curl_check() : void
    {
        if (!\extension_loaded('curl')) {
            /* translators: %s is replaced with "string" */
            $message = \sprintf(\__("Error: Please enable <a target='_blank' href='%s'>cURL extension</a> in PHP.", 'fintecture-payment-gateway'), "https://www.php.net/manual/en/book.curl.php");
            $this->add_admin_error($message);
        }
    }
    /**
     * Check if OpenSSL is available on the system
     *
     * @since 2.2
     */
    public function do_openssl_check() : void
    {
        if (!\extension_loaded('openssl')) {
            /* translators: %s is replaced with "string" */
            $message = \sprintf(\__("Error: Please enable <a target='_blank' href='%s'>OpenSSL extension</a> in PHP.", 'fintecture-payment-gateway'), "https://www.php.net/manual/fr/book.openssl.php");
            $this->add_admin_error($message);
        }
    }
    /**
     * Check if the 'plain' option is used in permalinks settings
     *
     * @since 2.2
     */
    public function do_permalinks_structure_check() : void
    {
        if (empty(\get_option('permalink_structure'))) {
            $message = \__("Error: Payments are not compatible with 'Plain' permalink structure. You could instead use the 'Post name' option, for example.", 'fintecture-payment-gateway');
            $this->add_admin_error($message);
        }
    }
    /**
     * Check the number of decimals used for prices
     *
     * @since 2.3
     */
    public function do_price_decimals_check() : void
    {
        $decimals = \wc_get_price_decimals();
        if ($decimals && $decimals !== 2) {
            $message = \__("Your price precision is different than two decimals. During paying of orders, we will round prices to two decimals.", 'fintecture-payment-gateway');
            $this->add_admin_error($message, 'warning');
        }
    }
    /**
     * Init PIS Client
     *
     * @return void
     * @since 3.0
     */
    private function initPisClient() : void
    {
        $wpContentDir = \dirname(\plugin_dir_path(__DIR__), 2) . '/';
        $environments = ['sandbox', 'production'];
        $currentEnv = $this->environment ? 'sandbox' : 'production';
        $isWooCommerce = \is_checkout() || \wp_doing_ajax();
        $isWebhook = \strpos($_SERVER['REQUEST_URI'], 'wc-api/fintecture_webhook') !== \false;
        foreach ($environments as $environment) {
            // Don't create more than one pisclient on front & in webhook
            if (($isWooCommerce || $isWebhook) && $currentEnv !== $environment) {
                continue;
            }
            $appId = $this->get_app_id($environment);
            $appSecret = $this->get_app_secret($environment);
            $privateKey = $this->get_private_key($environment);
            // Don't configure PISClient if we don't have all infos
            if (!$appId || !$appSecret || !$privateKey) {
                if (($appId || $appSecret || $privateKey) && $this->inFintectureSectionPage()) {
                    if ($environment === 'sandbox') {
                        \add_action('admin_notices', [$this, 'incompleteErrorSandbox']);
                    } else {
                        \add_action('admin_notices', [$this, 'incompleteErrorProduction']);
                    }
                }
                continue;
            }
            try {
                $this->pisClients[$environment] = new PisClient(['appId' => $appId, 'appSecret' => $appSecret, 'privateKey' => $privateKey, 'environment' => $environment, 'encryptionDir' => $wpContentDir]);
            } catch (\Exception $e) {
                if ($this->inFintectureSectionPage()) {
                    if ($environment === 'sandbox') {
                        \add_action('admin_notices', [$this, 'privateKeyErrorSandbox']);
                    } else {
                        \add_action('admin_notices', [$this, 'privateKeyErrorProduction']);
                    }
                } else {
                    return;
                }
            }
            if (isset($this->pisClients[$environment])) {
                // Get encrypted private key and check if we need to update CMS value
                $encryptedPrivateKey = $this->pisClients[$environment]->getEncryptedPrivateKey();
                if ($encryptedPrivateKey !== $privateKey) {
                    $option = $environment === 'sandbox' ? 'test_app_private_key' : 'app_private_key';
                    $this->update_option($option, $encryptedPrivateKey);
                }
            }
        }
    }
    /**
     * Returns PisClient depending of environment
     *
     * @param string $environment Environment
     * @param bool $generateToken Generate an access token
     * @param bool $log Log errors
     * @return PisClient|false
     * @since 1.0
     * @access private
     */
    private function getPisClient(string $environment = '', bool $generateToken = \true, bool $log = \true)
    {
        if (empty($environment)) {
            $environment = $this->environment ? 'sandbox' : 'production';
        }
        if (isset($this->pisClients[$environment])) {
            if ($generateToken) {
                $pisToken = $this->pisClients[$environment]->token->generate();
                /** @phpstan-ignore-line */
                if (!$pisToken->error) {
                    $this->pisClients[$environment]->setAccessToken($pisToken);
                    // set token of PIS client
                } else {
                    fintecture_write_log('Can\'t generate an access token: ' . $pisToken->errorMsg);
                    return \false;
                }
            }
            return $this->pisClients[$environment];
        }
        if ($log) {
            fintecture_write_log('PisClient for "' . $environment . '" does not exists');
        }
        return \false;
    }
    /**
     * Returns App ID depending of environment
     *
     * @param string $environment Environment
     * @return string
     * @since 1.0
     * @access private
     */
    private function get_app_id(string $environment = '') : string
    {
        if (empty($environment)) {
            $environment = $this->environment ? 'sandbox' : 'production';
        }
        return $environment === 'sandbox' ? $this->test_app_id : $this->app_id;
    }
    /**
     * Returns App Secret depending of environment
     *
     * @param string $environment Environment
     * @return string
     * @since 1.0
     * @access private
     */
    private function get_app_secret(string $environment = '') : string
    {
        if (empty($environment)) {
            $environment = $this->environment ? 'sandbox' : 'production';
        }
        return $environment === 'sandbox' ? $this->test_app_secret : $this->app_secret;
    }
    /**
     * Returns a Private Key depending of environment
     *
     * @param string $environment Environment
     * @return string
     * @since 1.0
     * @access private
     */
    private function get_private_key(string $environment = '') : string
    {
        if (empty($environment)) {
            $environment = $this->environment ? 'sandbox' : 'production';
        }
        return $environment === 'sandbox' ? $this->test_app_private_key : $this->app_private_key;
    }
    /**
     * Plugin options, this basically defines the configuration form
     *
     * @since 1.0
     */
    public function init_form_fields() : void
    {
        $this->form_fields = \apply_filters('wc_fintecture_settings', ['enabled' => ['title' => \__('Enable/Disable', 'fintecture-payment-gateway'), 'label' => \__('Enable this payment gateway', 'fintecture-payment-gateway'), 'type' => 'checkbox', 'description' => \__('Enable this module to include the Fintecture Payment Gateway.', 'fintecture-payment-gateway'), 'default' => 'no'], 'show_branding' => ['title' => \__('Fintecture Logo', 'fintecture-payment-gateway'), 'id' => 'fintecture-branding', 'label' => \__('Enable Fintecture Logo', 'fintecture-payment-gateway'), 'type' => 'checkbox', 'description' => \__('Show Fintecture logo at checkout.', 'fintecture-payment-gateway'), 'default' => 'yes'], 'environment' => ['title' => \__('Environment', 'fintecture-payment-gateway'), 'id' => 'fintecture-environment', 'label' => \__('Enable test mode (sandbox)', 'fintecture-payment-gateway'), 'type' => 'checkbox', 'description' => \__('Switch to production mode to start accepting real payments.', 'fintecture-payment-gateway'), 'default' => 'yes'], 'app_id' => ['title' => \__('App ID', 'fintecture-payment-gateway'), 'id' => 'fintecture-app-id', 'type' => 'text', 'description' => \__('This is the App ID provided to you when creating an application on Fintecture. <a href="https://console.fintecture.com/auth/register">Click here to create an account</a>', 'fintecture-payment-gateway')], 'app_secret' => ['title' => \__('App Secret', 'fintecture-payment-gateway'), 'id' => 'fintecture-app-secret', 'type' => 'password', 'description' => \__('This is the App Secret provided to you when creating an application on Fintecture. <a href="https://console.fintecture.com/auth/register">Click here to create an account</a>', 'fintecture-payment-gateway')], 'app_private_key' => ['title' => \__('Private Key', 'fintecture-payment-gateway'), 'id' => 'fintecture-app-private-key', 'type' => 'file_input', 'description' => \__('This is the Private Key provided to you when creating an application on Fintecture. <a href="https://console.fintecture.com/auth/register">Click here to create an account</a>', 'fintecture-payment-gateway'), 'value' => $this->app_private_key], 'test_production' => ['title' => \__('Check production credentials', 'fintecture-payment-gateway'), 'class' => 'fintecture-btn-test-prod-js button-secondary', 'type' => 'button', 'desc_tip' => \__('This allows you to verify your credentials.', 'fintecture-payment-gateway')], 'test_app_id' => ['title' => \__('Test (sandbox) App ID', 'fintecture-payment-gateway'), 'id' => 'fintecture-test-app-id', 'type' => 'text', 'description' => \__('This is the App ID provided to you when creating an application on Fintecture. <a href="https://console.fintecture.com/auth/register">Click here to create an account</a>', 'fintecture-payment-gateway')], 'test_app_secret' => ['title' => \__('Test (sandbox) App Secret', 'fintecture-payment-gateway'), 'id' => 'fintecture-test-app-secret', 'type' => 'password', 'description' => \__('This is the App Secret provided to you when creating an application on Fintecture. <a href="https://console.fintecture.com/auth/register">Click here to create an account</a>', 'fintecture-payment-gateway')], 'test_app_private_key' => ['title' => \__('Test (sandbox) Private Key', 'fintecture-payment-gateway'), 'id' => 'fintecture-test-app-private-key', 'type' => 'file_input', 'description' => \__('This is the Private Key provided to you when creating an application on Fintecture. <a href="https://console.fintecture.com/auth/register">Click here to create an account</a>', 'fintecture-payment-gateway'), 'value' => $this->test_app_private_key], 'test_sandbox' => ['title' => \__('Check test (sandbox) credentials', 'fintecture-payment-gateway'), 'class' => 'fintecture-btn-test-sandbox-js button-secondary', 'type' => 'button', 'desc_tip' => \__('This allows you to verify your credentials.', 'fintecture-payment-gateway')]]);
    }
    /**
     * Generate Button HTML
     * https://docs.woocommerce.com/document/implementing-wc-integration/
     *
     * @param string $key
     * @param mixed $data
     * @return string|false
     */
    public function generate_button_html(string $key, $data)
    {
        $field = $this->plugin_id . $this->id . '_' . $key;
        $defaults = ['class' => 'button-secondary', 'css' => '', 'custom_attributes' => [], 'desc_tip' => \false, 'description' => '', 'title' => ''];
        $data = \wp_parse_args($data, $defaults);
        \ob_start();
        require 'custom_wc_inputs/button.php';
        return \ob_get_clean();
    }
    /**
     * Generate File Input HTML
     * https://docs.woocommerce.com/document/implementing-wc-integration/
     *
     * @param string $key
     * @param mixed $data
     * @return string|false
     */
    public function generate_file_input_html(string $key, $data)
    {
        $field = $this->plugin_id . $this->id . '_' . $key;
        $defaults = ['class' => '', 'css' => '', 'custom_attributes' => [], 'desc_tip' => \false, 'description' => '', 'value' => ''];
        $data = \wp_parse_args($data, $defaults);
        \ob_start();
        require 'custom_wc_inputs/file_input.php';
        return \ob_get_clean();
    }
    /**
     * Admin save
     */
    public function process_admin_options()
    {
        // Empty admin notices to avoid duplicates
        \remove_action('admin_notices', [$this, 'admin_notices_fintecture']);
        \remove_action('admin_notices', [$this, 'privateKeyErrorSandbox']);
        \remove_action('admin_notices', [$this, 'privateKeyErrorProduction']);
        \remove_action('admin_notices', [$this, 'incompleteErrorSandbox']);
        \remove_action('admin_notices', [$this, 'incompleteErrorProduction']);
        $prefix = 'woocommerce_wc_fintecture_gateway_';
        $option1 = $prefix . 'app_private_key';
        // input file field 1
        $option2 = $prefix . 'test_app_private_key';
        // input file field 2
        $_POST[$option1] = $this->get_option('app_private_key');
        // default value for not loosing key at saving
        $_POST[$option2] = $this->get_option('test_app_private_key');
        // default value for not loosing key at saving
        if (!empty($_FILES)) {
            // if some files have been uploaded
            if (isset($_FILES[$option1]) && $this->checkUploadSuccess($_FILES[$option1])) {
                $app_private_key = \file_get_contents($_FILES[$option1]['tmp_name']);
                if (empty($app_private_key)) {
                    $this->add_settings_error('Production Private Key not valid. Please verify your .pem file.');
                }
                $_POST[$option1] = $app_private_key;
                // store the private key in the post var
            }
            if (isset($_FILES[$option2]) && $this->checkUploadSuccess($_FILES[$option2])) {
                $test_app_private_key = \file_get_contents($_FILES[$option2]['tmp_name']);
                if (empty($test_app_private_key)) {
                    $this->add_settings_error('Test (sandbox) Private Key not valid. Please verify your .pem file.');
                }
                $_POST[$option2] = $test_app_private_key;
                // store the test private key in the post var
            }
        }
        // Call the original method to save options (will read $this->get_post_data() again so that new POST values will be inside)
        parent::process_admin_options();
        try {
            Telemetry::logAction('save', fintecture_system_infos());
        } catch (\Exception $e) {
            fintecture_write_log('Can\'t send telemetry call: ' . $e);
        }
        return \true;
    }
    /**
     * Check if upload has worked
     *
     * @param array<mixed> $file Uploaded file
     * @return bool
     */
    private function checkUploadSuccess(array $file) : bool
    {
        if (isset($file['name']) && !empty($file['name'])) {
            if ($file['error'] === \UPLOAD_ERR_OK) {
                return \true;
            } else {
                $this->add_settings_error('App Private Key not uploaded:' . $file['error']);
            }
        }
        return \false;
    }
    /**
     * We are processing the payments here, if all is going well we change the order status to 'waiting payment' and redirect
     * to our payment gateway, if other case we update the order to fail.
     *
     * @param integer $order_id
     * @return array<string,string>|void redirection to our payment gateway
     * @since 1.0
     */
    public function process_payment($order_id)
    {
        // Verify permalink structure to avoid 404 error after connect redirection
        if (empty(\get_option('permalink_structure'))) {
            fintecture_write_log('\'Plain\' option used for permalink structure. Not supported by WooCommerce API.');
            $this->add_checkout_error();
            return;
        }
        // Order checks
        $order = \wc_get_order($order_id);
        if (!$order || \is_bool($order)) {
            fintecture_write_log('Payment failed: order not found');
            $this->add_checkout_error();
            return;
        } elseif (!\is_a($order, 'WC_Order')) {
            fintecture_write_log('The order type is not WC_Order');
            $this->add_checkout_error();
            return;
        }
        if (0 >= $order->get_total()) {
            return $this->complete_free_order($order);
        }
        try {
            // Mark as on-hold (we're awaiting the payment)
            $order->update_status('pending_payment', \__('Awaiting payment', 'fintecture-payment-gateway'));
            $payload = $this->get_payload($order);
            try {
                $state = Crypto::encodeToBase64(['order_id' => $order->get_id()]);
                $pisClient = $this->getPisClient();
                if (!$pisClient) {
                    $this->add_checkout_error();
                    return;
                }
                $connect = $pisClient->connect->generate($payload, $state, null, \wc_get_checkout_url());
                /** @phpstan-ignore-line */
                if (!$connect->error) {
                    return ['result' => 'success', 'redirect' => $connect->meta->url];
                } else {
                    fintecture_write_log('Error: ' . $connect->errorMsg);
                    $this->add_checkout_error();
                    return;
                }
            } catch (\Exception $e) {
                fintecture_write_log('Error: ' . $e->getMessage());
                $this->add_checkout_error();
                return;
            }
        } catch (\Exception $e) {
            $order->update_status('failed');
            fintecture_write_log('Error: ' . $e->getMessage());
            $this->add_checkout_error();
            return;
        }
    }
    /**
     * Completes an order without a positive value. We contemplate this case as a free payment
     *
     * @param mixed $order The order to complete
     * @return array<string> Redirection data for 'process_payment'
     * @since 1.0
     * @access private
     */
    private function complete_free_order($order) : array
    {
        $order->payment_complete();
        /** @phpstan-ignore-next-line */
        get_wc_instance()->cart->empty_cart();
        // Remove cart
        // Return thank you page redirect.
        return ['result' => 'success', 'redirect' => $this->get_return_url($order)];
    }
    /**
     * Generates an array with the payload information
     *
     * @param mixed $order
     * @return array<string, array<string, array<string, string>|string>>
     * @since 1.0
     * @access private
     */
    private function get_payload($order) : array
    {
        $total = $order->get_total();
        $total = \round($total, 2);
        // keep only 2 decimals
        $currency_code = $order->get_currency();
        $billing_full_name = $order->get_formatted_billing_full_name();
        $billing_email = $order->get_billing_email();
        $billing_phone = $order->get_billing_phone();
        $psu_address_street = $order->get_billing_address_1();
        $psu_address_complement = $order->get_billing_address_2();
        $psu_address_zip = $order->get_billing_postcode();
        $psu_address_city = $order->get_billing_city();
        $psu_address_country = $order->get_billing_country();
        $psu_company = $order->get_billing_company();
        return ['data' => ['type' => 'PAYMENT', 'attributes' => ['amount' => (string) $total, 'currency' => $currency_code, 'communication' => \FINTECTURE_PAYMENT_PREFIX . $order->get_id() . '']], 'meta' => ['psu_name' => \sanitize_text_field($billing_full_name), 'psu_email' => \sanitize_email($billing_email), 'psu_ip' => $order->get_customer_ip_address(), 'psu_phone' => \sanitize_text_field($billing_phone), 'psu_company' => \sanitize_text_field($psu_company), 'psu_address' => ['street' => \sanitize_text_field($psu_address_street), 'complement' => \sanitize_text_field($psu_address_complement), 'zip' => \sanitize_text_field($psu_address_zip), 'city' => \sanitize_text_field($psu_address_city), 'country' => \sanitize_text_field($psu_address_country)]]];
    }
    /**
     * After the payment, gateway returns to WooCommerce and executes this function.
     * This function evaluates if the payment was successful or not by checking the
     * params of the URL and checks if some parameter was changed to make a fraud
     * attempt by means a digest code verification
     *
     * @return array<string, string>|void
     * @since 1.0
     */
    public function post_payment()
    {
        [$order, $order_id, $session_id] = fintecture_post_payment();
        if (!$order || !$order_id || !$session_id) {
            $this->redirect_checkout_error();
        }
        $pisClient = $this->getPisClient();
        if ($pisClient) {
            $payment = $pisClient->payment->get($session_id);
            /** @phpstan-ignore-line */
            if (!$payment->error) {
                $communicationMatch = $payment->data->attributes->communication === \FINTECTURE_PAYMENT_PREFIX . $order_id;
                $payment_status = $communicationMatch ? $payment->meta->status : 'payment_unsuccessful';
            }
        }
        if (!isset($payment_status)) {
            if (isset($payment)) {
                fintecture_write_log('Error: ' . $payment->errorMsg);
            }
            $this->redirect_checkout_error();
        } elseif ($payment_status === 'payment_created') {
            // Some notes to customer (replace true with false to make it private)
            $order->add_order_note(\__('Payment created. Thank you for your trust. Fintecture team', 'fintecture-payment-gateway'), \true);
            $order->payment_complete($session_id);
            \add_post_meta($order->get_id(), '_transaction_id', $session_id, \true);
            get_wc_instance()->cart->empty_cart();
            /** @phpstan-ignore-line */
            $order->update_status('processing');
            do_action('fintecture_payment_created', $order_id);
        } elseif ($payment_status === 'payment_unsuccessful' || $payment_status === 'sca_required') {
            $this->redirect_checkout_unsuccessful_payment_error();
        } elseif ($payment_status === 'payment_pending') {
            $order->add_order_note(\__('Payment pending. Thank you for your trust. Fintecture team', 'fintecture-payment-gateway'), \true);
            get_wc_instance()->cart->empty_cart();
            /** @phpstan-ignore-line */
            $order->update_status('on-hold');
        } else {
            fintecture_write_log('Failed payment for order: ' . $order_id);
            $order->update_status('failed');
            $this->redirect_checkout_unsuccessful_payment_error();
        }
        // Redirect to the thank you page
        \wp_redirect($order->get_checkout_order_received_url());
        exit;
    }
    /**
     * A few moment after the payment, gateway returns to WooCommerce and executes this function
     * This function set the final order status to be sure to have the most up-to-date status
     */
    public function webhook() : void
    {
        if (!isset($_SERVER['HTTP_SIGNATURE']) || !isset($_SERVER['HTTP_DIGEST'])) {
            fintecture_write_log('webhook - invalid_headers');
            \http_response_code(400);
            exit('invalid_headers');
        } elseif (!$this->validSignature()) {
            fintecture_write_log('webhook - invalid_signature');
            \http_response_code(401);
            exit('invalid_signature');
        }
        $response = fintecture_webhook();
        $status = $response['params']['status'];
        $order = $response['order'];
        if (!$order) {
            fintecture_write_log('webhook - no_order');
            \http_response_code(400);
            exit('no_order');
        } elseif (!$status) {
            fintecture_write_log('webhook - no_status');
            \http_response_code(400);
            exit('no_status');
        }
        if ($order->get_status() === 'pending' || $order->get_status() === 'on-hold' || $order->get_status() === 'failed') {
            if ($status === 'payment_created') {
                $order->update_status('processing');
                do_action('fintecture_payment_created', $order->get_id());
            } elseif ($status === 'payment_pending') {
                $order->update_status('on-hold');
            } elseif ($status === 'payment_error') {
                $order->update_status('failed');
            } elseif ($status === 'payment_unsuccessful') {
                $order->update_status('payment_pending');
            } else {
                exit('no_update');
            }
        } else {
            exit('no_update_needed');
        }
        exit;
    }
    /**
     * Endpoint for testing credentials
     */
    public function connection_test() : void
    {
        $private_key = $_POST['private_key'] ?? '';
        if (empty($private_key)) {
            $pisClient = $this->getPisClient($_POST['environment'], \false, \false);
            if ($pisClient) {
                $private_key = $pisClient->getFinalPrivateKey();
            }
        }
        if (isset($_POST['app_id']) && !empty($_POST['app_id']) && isset($_POST['app_secret']) && !empty($_POST['app_secret']) && !empty($private_key)) {
            $response = Validation::validCredentials('pis', ['appId' => $_POST['app_id'], 'appSecret' => $_POST['app_secret'], 'privateKey' => $private_key], $_POST['environment']);
            $code = $response ? 200 : 400;
        } else {
            $code = 400;
        }
        \http_response_code($code);
        exit;
    }
    /**
     * Endpoint to send checkout event
     */
    public function checkout_event() : void
    {
        $log = Telemetry::logAction('checkout', fintecture_system_infos());
        $code = $log ? 200 : 400;
        \http_response_code($code);
        exit;
    }
    /**
     * Endpoint to send metric event (checkout dropdown)
     */
    public function dropdown_event() : void
    {
        $log = Telemetry::logMetric('custom-module-checkout-tooltip-displayed');
        $code = $log ? 200 : 400;
        \http_response_code($code);
        exit;
    }
    /**
     * Check if a signature is valid
     *
     * @return bool
     */
    private function validSignature() : bool
    {
        $body = \file_get_contents('php://input');
        $digest = \stripslashes($_SERVER['HTTP_DIGEST']);
        $signature = \stripslashes($_SERVER['HTTP_SIGNATURE']);
        return Validation::validSignature($body, $digest, $signature);
    }
    /**
     * Return the gateway's icon.
     *
     * @return string
     */
    public function get_icon()
    {
        $icon = $this->show_branding ? '<img src="https://assets.fintecture.com/img/new-checkout/woocommerce/fintecture.svg" style="margin-top: 5px;" alt="' . \esc_attr($this->get_title()) . '">' : '';
        return \apply_filters('woocommerce_gateway_icon', $icon, $this->id);
    }
    /**
     * Create the checkout template
     *
     * @return void
     */
    public function payment_fields()
    {
        $template_args = ['img_path' => \plugin_dir_url(__FILE__) . '../assets/img/'];
        echo \wc_get_template('checkout-description.php', $template_args, '', \plugin_dir_path(__FILE__) . '../views/');
        // we can't use our notice function for this hook
        if (\strpos($_SERVER['REQUEST_URI'], '?err=request_error') !== \false) {
            \wc_add_notice(\__('An error occurred while processing your request.', 'fintecture-payment-gateway'), 'error');
        } elseif (\strpos($_SERVER['REQUEST_URI'], '?err=unsuccessful_payment') !== \false) {
            \wc_add_notice(\__('The payment was either cancelled or was unsuccessful. Please try again.', 'fintecture-payment-gateway'), 'error');
        }
    }
    /**
     * Redirect user to checkout page with a generic error
     */
    private function redirect_checkout_error() : void
    {
        \wp_safe_redirect(\wc_get_checkout_url() . '?err=request_error');
        exit;
    }
    /**
     * Redirect user to checkout page with an unsuccessful payment error
     */
    private function redirect_checkout_unsuccessful_payment_error() : void
    {
        \wp_safe_redirect(\wc_get_checkout_url() . '?err=unsuccessful_payment');
        exit;
    }
    /**
     * Redirect user to checkout page with an error
     */
    private function add_checkout_error() : void
    {
        $this->add_front_error(\__('An error occurred while processing your request.', 'fintecture-payment-gateway'));
    }
}
\class_alias('FintectureWC\\Fintecture_Post_Payment', 'Fintecture_Post_Payment', \false);
